// sphere.C
// Fabian Roth, fin@envoy.ch, 2003

#include "sphere.h"


Vertex *Edge::getHalfVertex()  {
  doDivide();
  return v12;
}

Edge *Edge::getSubEdge(int i)  {

  doDivide();

  if (i)
    return e2;
  else
    return e1;
}

void Edge::doDivide() {

  // already divided?
  if (v12)
    return;

  float x = (v1->getx() + v2->getx()) / 2.0f;
  float y = (v1->gety() + v2->gety()) / 2.0f;
  float z = (v1->getz() + v2->getz()) / 2.0f;
  
  v12 = new Vertex(x,y,z,generation+1);

  e1 = new Edge(v1, v12, generation+1);
  e2 = new Edge(v12, v2, generation+1);

}


void Face::doDivide() {

  // already divided?
  if (f1) 
    return;

  // need a few more edges:

  Edge *e1m = new Edge(e1->getHalfVertex(), e3->getHalfVertex(),generation+1);
  Edge *e2m = new Edge(e2->getHalfVertex(), e1->getHalfVertex(),generation+1);
  Edge *e3m = new Edge(e3->getHalfVertex(), e2->getHalfVertex(),generation+1);

  f1 = new Face(e1->getSubEdge(o1),o1,e1m,0,e3->getSubEdge(!o3),o3, generation+1);
  f2 = new Face(e1->getSubEdge(!o1),o1,e2->getSubEdge(o2),o2, e2m,0, generation+1);
  f3 = new Face(e2->getSubEdge(!o2),o2,e3->getSubEdge(o3),o3, e3m,0, generation+1);
  f4 = new Face(e1m,1,e2m,1,e3m,1, generation+1);
}

void Face::doDivision() {
  
  for (faceIterator iter = all.begin(); iter!=all.end(); iter++)
    (*iter)->doDivide();

  acceptNew();
  Edge::acceptNew();

}



vector<Vertex *> Vertex::all;
int Vertex::allIndex = 0;

vector<Edge *> Edge::all;
vector<Edge *> Edge::allNew;
int Edge::allIndex = 0;

vector<Face *> Face::all;
vector<Face *> Face::allNew;
int Face::allIndex = 0;


void initTetrahedron() {

  Vertex *v1 = new Vertex(-1,1,-1);
  Vertex *v0 = new Vertex(1,-1,-1);
  Vertex *v2 = new Vertex(-1,-1,1);
  Vertex *v3 = new Vertex(-1,-1,-1);

  Edge *e0 = new Edge(v0, v1);
  Edge *e1 = new Edge(v1, v2);
  Edge *e2 = new Edge(v2, v0);
  Edge *e3 = new Edge(v0, v3);
  Edge *e4 = new Edge(v3, v2);
  Edge *e5 = new Edge(v3, v1);

  Face *f0 = new Face(e0,0,e1,0,e2,0);
  Face *f1 = new Face(e1,1,e5,1,e4,0);
  Face *f2 = new Face(e2,1,e4,1,e3,1);
  Face *f3 = new Face(e3,0,e5,0,e0,1);

  Face::acceptNew();
  Edge::acceptNew();

}


void initOctahedron() {

  Vertex *v0 = new Vertex(0,1,0);
  Vertex *v1 = new Vertex(-1,0,1);
  Vertex *v2 = new Vertex(1,0,1);
  Vertex *v3 = new Vertex(1,0,-1);
  Vertex *v4 = new Vertex(-1,0,-1);
  Vertex *v5 = new Vertex(0,-1,0);

  Edge *e0 = new Edge(v0,v1);
  Edge *e1 = new Edge(v1,v2);
  Edge *e2 = new Edge(v2,v0);
  Edge *e3 = new Edge(v1,v4);
  Edge *e4 = new Edge(v4,v0);
  Edge *e5 = new Edge(v4,v3);
  Edge *e6 = new Edge(v3,v0);
  Edge *e7 = new Edge(v3,v2);
  Edge *e8 = new Edge(v2,v5);
  Edge *e9 = new Edge(v5,v1);
  Edge *e10 = new Edge(v4,v5);
  Edge *e11 = new Edge(v3,v5);
  
  Face *f0 = new Face(e0,0,e1,0,e2,0);
  Face *f1 = new Face(e0,1,e4,1,e3,1);
  Face *f2 = new Face(e4,0,e6,1,e5,1);
  Face *f3 = new Face(e6,0,e2,1,e7,1);
  Face *f4 = new Face(e1,1,e9,1,e8,1);
  Face *f5 = new Face(e9,0,e3,0,e10,0);
  Face *f6 = new Face(e10,1,e5,0,e11,0);
  Face *f7 = new Face(e11,1,e7,0,e8,0);

  Face::acceptNew();
  Edge::acceptNew();
}

void initIcosahedron() {

  float x = .525731112119133606;
  float z = .850650808352039932;


  Vertex *v0 = new Vertex(.0,-z,.0); 
  Vertex *v1 = new Vertex(-x,-x,-x);  
  Vertex *v2 = new Vertex(-x,-x,x);
  Vertex *v3 = new Vertex(x,-x,z); 
  Vertex *v4 = new Vertex(z,-x,0);  
  Vertex *v5 = new Vertex(x,-x,-z); 
  Vertex *v6 = new Vertex(-x,x,z); 
  Vertex *v7 = new Vertex(x,x,x); 
  Vertex *v8 = new Vertex(x,x,-x);  
  Vertex *v9 = new Vertex(-x,x,-z); 
  Vertex *v10 = new Vertex(-z,x,.0); 
  Vertex *v11 = new Vertex(-0,z,0);

//   Vertex *v0 = new Vertex(-x,.0,z); 
//   Vertex *v1 = new Vertex(x,.0,z);  
//   Vertex *v2 = new Vertex(-x,.0,-z);
//   Vertex *v3 = new Vertex(x,.0,-z); 
//   Vertex *v4 = new Vertex(.0,z,x);  
//   Vertex *v5 = new Vertex(.0,z,-x); 
//   Vertex *v6 = new Vertex(.0,-z,x); 
//   Vertex *v7 = new Vertex(0,-z,-x); 
//   Vertex *v8 = new Vertex(z,x,.0);  
//   Vertex *v9 = new Vertex(-z,x,.0); 
//   Vertex *v10 = new Vertex(z,-x,.0); 
//   Vertex *v11 = new Vertex(-z,-x,.0);
		
  		

  Edge *e0 = new Edge(v2, v0);
  Edge *e1 = new Edge(v0, v3);
  Edge *e2 = new Edge(v3, v2);
  Edge *e3 = new Edge(v4, v0);
  Edge *e4 = new Edge(v3, v4);
  Edge *e5 = new Edge(v4, v5);
  Edge *e6 = new Edge(v5, v0);
  Edge *e7 = new Edge(v0, v1);
  Edge *e8 = new Edge(v1, v5);
  Edge *e9 = new Edge(v1, v2);
  Edge *e10 = new Edge(v2, v6);
  Edge *e11 = new Edge(v6, v3);
  Edge *e12 = new Edge(v3, v7);
  Edge *e13 = new Edge(v7, v4);
  Edge *e14 = new Edge(v4,  v8);
  Edge *e15 = new Edge(v8, v5);
  Edge *e16 = new Edge(v5, v9);
  Edge *e17 = new Edge(v9, v1);
  Edge *e18 = new Edge(v1, v10);
  Edge *e19 = new Edge(v10, v2);
  Edge *e20 = new Edge(v10, v6);
  Edge *e21 = new Edge(v6, v7);
  Edge *e22 = new Edge(v7, v8);
  Edge *e23 = new Edge(v8, v9);
  Edge *e24 = new Edge(v9, v10);
  Edge *e25 = new Edge(v11, v9);
  Edge *e26 = new Edge(v11, v10);
  Edge *e27 = new Edge(v11, v6);
  Edge *e28 = new Edge(v11, v7);
  Edge *e29 = new Edge(v11,v8);

  Face *f0 = new Face(e0,0,e1,0,e2,0);
  Face *f1 = new Face(e1,1,e3,1,e4,1);
  Face *f2 = new Face(e3,0,e6,1,e5,1);
  Face *f3 = new Face(e6,0,e7,0,e8,0);
  Face *f4 = new Face(e7,1,e0,1,e9,1);
  Face *f5 = new Face(e19,0,e10,0,e20,1);
  Face *f6 = new Face(e10,1,e2,1,e11,1);
  Face *f7 = new Face(e11,0,e12,0,e21,1);
  Face *f8 = new Face(e12,1,e4,0,e13,1);
  Face *f9 = new Face(e13,0,e14,0,e22,1);
  Face *f10 = new Face(e14,1,e5,0,e15,1);
  Face *f11 = new Face(e15,0,e16,0,e23,1);
  Face *f12 = new Face(e16,1,e8,1,e17,1);
  Face *f13 = new Face(e17,0,e18,0,e24,1);
  Face *f14 = new Face(e18,1,e9,0,e19,1);
  Face *f15 = new Face(e20,0,e27,1,e26,0);
  Face *f16 = new Face(e27,0,e21,0,e28,1);
  Face *f17 = new Face(e28,0,e22,0,e29,1);
  Face *f18 = new Face(e29,0,e23,0,e25,1);
  Face *f19 = new Face(e25,0,e24,0,e26,1);

  Face::acceptNew();
  Edge::acceptNew();

}
