// sphere.h
// Fabian Roth, fin@envoy.ch, 2003


#include <vector>
#include <iostream>

#include <stdlib.h>
#include <math.h>

class Vertex {

  int index;
  int generation;
  float x,y,z;
  
public:

  Vertex(float _x, float _y, float _z, int _generation = 0) {
    x = _x;
    y = _y;
    z = _z;
    generation = _generation;

    all.push_back(this);
    index=allIndex++;

    normalize();
    //    disturb();

  }

  float getx() {
    return x;
  }
  float gety() {
    return y;
  }
  float getz() {
    return z;
  }
  int getIndex() {
    return index;
  }

  void disturb() {
    int r = random()%100;
    float disturb = (float)r/200.0;
    float abs = sqrt(x*x + y*y +z*z);
    abs *= disturb;
    x += disturb;
    y -= disturb;
    //    z /= abs;
  }


  void normalize() {
    float abs = sqrt(x*x + y*y + z*z);
    x /= abs;
    y /= abs;
    z /= abs;
  }

  static vector<Vertex *> all;  
  typedef vector<Vertex *>::iterator vertexIterator;
  static int allIndex;

  void print(ostream &o) {
    o << index << '('<<generation<<"):" << x << ',' << y << ',' << z << '\n';
  }

  void dump(ostream &o) {
    float arr[4];
    arr[0] = x;
    arr[1] = y;
    arr[2] = z;
    arr[3] = 1.0;

    o.write((char *)arr, 4*sizeof(float));
  }

  void space(ostream &o) {
    float arr[4];
    arr[0] = .0;
    arr[1] = .0;
    arr[2] = .0;
    arr[3] = .0;

    o.write((char *)arr, 4*sizeof(float));
  }

  static void printAll(ostream &o) {
    o<<"Vertices\n";
    for (vertexIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->print(o);
  }

  static void dumpAll(ostream &o) {
    cout << "dumping "<< all.size()<<" vertices\n";
    for (vertexIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->dump(o);
  }

  static void spaceAll(ostream &o) {
    cout << "spacing "<< all.size()<<" vertices\n";
    for (vertexIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->space(o);
  }




};

class Edge {
  
  int index;
  int generation;
  Vertex *v1,*v2;

  Vertex *v12;
  Edge *e1, *e2;

  void doDivide();

public:

  Edge(Vertex *_v1, Vertex *_v2, int _generation = 0) {
    v1 = _v1;
    v2 = _v2;
    generation = _generation;

    v12=0;
    e1=e2=0;

    allNew.push_back(this);
    index = allIndex++;


  }

  Vertex *getVertex(int i) {
    if (i) 
      return v2;
    else 
      return v1;
  }

  int getIndex() {
    return index;
  }

  static vector<Edge *> all;
  static vector<Edge *> allNew;
  typedef vector<Edge *>::iterator edgeIterator;
  static int allIndex;


  Vertex *getHalfVertex();
  Edge *getSubEdge(int i);

  void print(ostream &o) {
    o << index << '('<<generation<<"):" << v1->getIndex() << ',' << v2->getIndex()  << '\n';
  }

  void dump(ostream &o) {
    int arr[4];
    arr[0]=v1->getIndex();
    arr[1]=v2->getIndex();
    arr[2]=0;
    arr[3]=0;
    
    o.write((char *)arr, 4*sizeof(int));
  }

  static void printAll(ostream &o) {
    o<<"Edges\n";
    for (edgeIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->print(o);
  }

  static void dumpAll(ostream &o) {
    cout << "dumping "<< all.size()<<" edges\n";
    for (edgeIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->dump(o);
  }

  static void acceptNew() {
    all = allNew;
    allNew.clear();
  }



};

class Face {

  int index;
  int generation;
  
  Edge *e1, *e2, *e3;
  // orientations of edges
  int o1, o2, o3;

  Face *f1, *f2, *f3, *f4;

  void doDivide();


public:

  Face(Edge *_e1, int _o1, Edge *_e2, int _o2, Edge *_e3, int _o3, int _generation =0) {
    e1 = _e1;
    e2 = _e2;
    e3 = _e3;
    o1 = _o1;
    o2 = _o2;
    o3 = _o3;
    generation = _generation;

    f1=f2=f3=f4=0;

    allNew.push_back(this);
    index = allIndex++;
  }

  static vector<Face *> all;
  static vector<Face *> allNew;
  typedef vector<Face *>::iterator faceIterator;
  static int allIndex;

  static void doDivision();

  void print(ostream &o) {
    o << index << '('<<generation<<"):" << e1->getIndex() << ',' << e2->getIndex() << ',' << e3->getIndex() << '\n';
  }

  void printVertex(ostream &o) {
    o << index << '('<<generation<<"):" << e1->getVertex(o1)->getIndex() << ',' << e2->getVertex(o2)->getIndex() << ',' << e3->getVertex(o3)->getIndex() << '\n';
  }

  void dump(ostream &o) {
    int arr[4];
    arr[0]=e1->getVertex(o1)->getIndex();
    arr[1]=e2->getVertex(o2)->getIndex();
    arr[2]=e3->getVertex(o3)->getIndex();
    arr[3]=0;
    
    o.write((char *)arr, 4*sizeof(int));
  }


  static void printAll(ostream &o) {
    o<<"Faces\n";
    for (faceIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->print(o);
  }

  static void printAllVertex(ostream &o) {
    o<<"Faces\n";
    for (faceIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->printVertex(o);
  }

  static void dumpAll(ostream &o) {
    cout << "dumping " << all.size() << " faces\n";
    for (faceIterator iter = all.begin(); iter != all.end();iter++)
      (*iter)->dump(o);
  }

  
  static void acceptNew() {
    all = allNew;
    allNew.clear();
  }
  


};


void initTetrahedron();
void initOctahedron();
void initIcosahedron();
