
;---
; Kalleidoscope Stage1.vcl
; Random move all the seed triangles
;---


kScrCenterX .assign 2048
kScrCenterY .assign 2048
kTriSize  .assign 25

kGifPacket .assign 0

kTriangleStart .assign 1
kDamp		.assign	961
kDoubleHalf	.assign 962
kVelocityStart .assign 963
kInited .assign 983
kRound	.assign 984
kRseed  .assign 985


.init_vf_all
.init_vi_all
.syntax new

.vu

--enter
--endenter


; macro to load immediate to a register

	.macro	loixyzw	reg,xval,yval,zval,wval
	loi		\xval
	addi.x		\reg,vf00,i

	loi		\yval
	addi.y		\reg,vf00,i

	loi		\zval
	addi.z		\reg,vf00,i

	loi		\wval
	muli.w		\reg,vf00,i
	.endm

; macro to apply the velocity onto one vertex
	.macro	apply_velocity iCoord, fCoord, fScrCenter, fVelocity

	move.xyzw	\fCoord, vf00
	lq.xyz	\fCoord, (\iCoord)
	itof4.xyz	\fCoord,	\fCoord		;convert fixed point to float point
	sub.xyz	\fCoord, \fCoord, \fScrCenter	;convert to world coordinate
		
	add.xyz	\fCoord,	\fCoord,	\fVelocity	; add dx,dy,dz
	
	.endm

; macro to move vertex
	.macro	move_vertex iCoord, fCoord, fScrCenter, fVelocity, fDir

	iaddiu	\iCoord, \iCoord,	2		; next vertex
	mul.xyz fTemp1,	\fCoord,   \fDir		; mirror coord
	mul.xyz	fTemp2, \fVelocity, \fDir		; mirror velocity
	
	add.xyz		fTemp1,	fTemp1, fTemp2
	add.xyz		fTemp1,	fTemp1, \fScrCenter
	ftoi4.xyz	fTemp1, fTemp1
	sq.xy		fTemp1, (\iCoord)	; Update the screen coordinate
		
	.endm	

; macro to do bound check
	.macro bound_chk fcoord, bound, cond, bounce, cont
	
	fcset 		0  				; reset clipping stuff
	iaddiu		ibound,vi00,\bound	
	mfir.w		fbound,ibound
	itof0.w		fbound,fbound
	clipw.xyz	\fcoord,fbound
	fcand		vi01, \cond 		
	ibgtz		vi01, \bounce
	b \cont
	
	.endm

; macro to swap x,y elements of float vector
	.macro swap_xy vector
	
	mr32.xyzw	\vector, \vector 	; x <= y
	move.x		fResult, \vector
	mr32.xyzw	\vector, \vector
	mr32.xyzw	\vector, \vector	; y <= x
	move.y		fResult, \vector
	move.xy		\vector, fResult

	.endm

; macro to generate rand
	.macro	gen_rand
	
	mula		rSeed,rSeed,rMul   
	maddw		rSeed,rAdd,vf00
	ftoi0		rI,rSeed
	itof0		rI,rI
	sub		rSeed,rSeed,rI			; rSeed now [0-1.0)

	.endm

START:

; initialize registers
	
	iaddiu	itemp, vi00, 2048		; Screen center
	move.xyzw	fScrCenter,	vf00
	mfir.xy	fScrCenter,	itemp
	itof0.xyzw	fScrCenter, fScrCenter

	iaddi		iminus1, vi00, -1	
	iaddi		iplus1,	vi00, 1	
	
	mfir.xyzw	fPlus1, iplus1
	itof0.xyzw	fPlus1, fPlus1
	mfir.yzw	fXminus1, iplus1              
	mfir.x		fXminus1, iminus1
	itof0.xyzw	fXminus1, fXminus1             
   	mfir.xzw	fYminus1, iplus1              
	mfir.y		fYminus1, iminus1
	itof0.xyzw	fYminus1, fYminus1             
	mfir.zw		fXYminus1, iplus1              
	mfir.xy		fXYminus1, iminus1
	itof0.xyzw	fXYminus1, fXYminus1             
   

	lq.xyzw	fDamp,	kDamp(vi00)

	lq.xyzw fXdoubleYhalf,	kDoubleHalf(vi00)

	iaddiu  iDataindex, vi00, kRound
	ilwr.x	iTemp,	(iDataindex)
	iaddiu	iTemp2,	vi00,	650
	ibne	iTemp,	iTemp2, cont1	

	; Re-generate Triangles after 650 rounds
	iswr.x	vi00,	(iDataindex)   ; clear rounds
	b init
	
cont1:
	iaddiu	iTemp,	iTemp,	1
	iswr.x	iTemp,	(iDataindex)

	iaddiu	iDataindex, vi00, kInited	
	ilwr.x iInitted, (iDataindex)
	ibgtz iInitted, prepare_move  ;if inited, skip initialization

	
	loixyzw		rSeed,1.31357231,1.79512412,17.2341241,3.214341
	sq.xyzw	rSeed, kRseed(vi00)
; ----------------------------------------------------
; Init the twenty primary triangles
init:

	iaddiu		itemp,vi00, 2			; random scale
	mfir.xyzw	frandscale,itemp
	itof0.xyzw	frandscale,frandscale

	iaddiu		itemp,vi00, 1			; random offset
	mfir.xyzw	frandoffs,itemp
	itof0.xyzw	frandoffs,frandoffs

	; random seed init values	
	lq.xyzw		rSeed, kRseed(vi00)
	loixyzw		rMul,1.31357231,1.79512412,17.2341241,3.214341
	loixyzw		rAdd,0.71234121,0.1245121,0.313815147,0.6185812

	iaddiu iTriCount, vi00, 20
	iaddiu	iCoord,	vi00, kTriangleStart   ;point to 1st triangle	
	iaddiu	iCoord, iCoord, 1 	;point to the coord of 1st vertex
	
init_triangle:     		

	; -------------- First Vertex --------------------------
	gen_rand

	; set ftemp to (1,1,1,1)
	move.xyzw	fCoord, fPlus1

	mul.xy		fCoord,rSeed,frandscale     ; (0, 2)
	sub.xy		fCoord,fCoord,frandoffs      ; (-1, 1)

	iaddiu		itemp, vi00, 105
	mfir.y		fdrandrange,itemp
	iaddiu		itemp, vi00, 210
	mfir.x		fdrandrange,itemp
	itof0.xy	fdrandrange,fdrandrange

	mul.xy  	fCoord,fCoord,fdrandrange    ; y:[-105, +105], x:[-210, +210]
	move.xyzw	fTemp, fCoord
	add.xyz		fTemp, fTemp, fScrCenter
	ftoi4.xyzw	fTemp,	fTemp
	
	sq.xyzw    	fTemp,(iCoord)		; store the vertex

	; -------------- Second Vertex --------------------------
	iaddiu		iCoord, iCoord, 2
	gen_rand

	; set ftemp to (1,1,1,1)
	move.xyzw	fTemp, fPlus1	
	
	iaddiu		itemp,vi00, kTriSize		; size of triangle
	mfir.y		fdrandrange,itemp
	iaddiu		itemp,vi00, kTriSize*2
	mfir.x		fdrandrange, itemp
	itof0.xy	fdrandrange,fdrandrange

	mul.xy  	fTemp,rSeed,fdrandrange    ; dx, dy should be within (0, 15)
	add.xy		fTemp, fCoord, fTemp
	add.xyz		fTemp, fTemp,  fScrCenter
	ftoi4.xyzw	fTemp, fTemp

	sq.xyzw    	fTemp,(iCoord)		; store the vertex

	; -------------- Third Vertex --------------------------
	iaddiu		iCoord, iCoord, 2
	gen_rand

	; set ftemp to (1,1,1,1)
	move.xyzw	ftemp, fPlus1	
	
	mul.xy  	fTemp,rSeed,fdrandrange    ; dx, dy should be within (0, 15)
	add.xy		fTemp, fCoord, fTemp
	add.xyz		fTemp, fTemp,  fScrCenter
	ftoi4.xyzw	fTemp, fTemp

	sq.xyzw    	fTemp,(iCoord)		; store the vertex

	; skip the next 7 triangles
	iaddiu		iCoord, iCoord, 44

	; all triangles?
	isubiu		iTricount,iTricount,1     
	ibne		iTricount,vi00, init_triangle ; if iTricount!=0 goto next triangle


; Init velocity of each of the twenty primary triangles
pre_velocity:
	iaddiu		itemp,vi00, 2			; dy  range
	mfir.y		fdrandrange,itemp
	iaddiu		itemp,vi00, 4			; dx range
	mfir.x		fdrandrange,itemp
	itof0.xy	fdrandrange,fdrandrange

	iaddiu iTriCount, vi00, 20
	iaddiu iDataindex, vi00, kVelocityStart

init_triangle_velocity:     		

	gen_rand

	; convert 0-1.0 to the kind of numbers we want

	; set ftemp to (0,0,0,0)
	move.xyzw	ftemp, vf00

	mul.xy		ftemp,rSeed,frandscale
	sub.xy		ftemp,ftemp,frandoffs      ; +1 or -1
	mul.xy  	ftemp,ftemp,fdrandrange    ; dy:(-2,+2)  dx:(-4,+4)

	sqi.xy    	ftemp,(iDataindex++)		; store the velociy dx,dy

	; all triangles?
	isubiu		iTricount,iTricount,1     
	ibne		iTricount,vi00, init_triangle_velocity	; if iTricount!=0 goto next triangle



	; done initting velocities
	sq.xyzw	rSeed,	kRseed(vi00)   ; remember current rSeed

	iaddiu	iDataindex, vi00, kInited
	iaddiu	itemp, vi00, 1
	iswr.x itemp, (iDataindex)	;set initted flag=1, 
					;don't init velocity next time

prepare_move:
	iaddiu	iCoord,	vi00, kTriangleStart   ;point to 1st triangle
	iaddiu	iCoord, iCoord, 1 	;point to the coord of 1st vertex
	iaddiu iVelocity, vi00,	kVelocityStart ;point to veclocity of 1st prim triangle

	move.xyzw	fVelocity, vf00
	move.xyzw	fCoord,	 vf00

	iaddiu	iCounter0,	vi00,	20	;20 primary triangles
	
move_pri_tri:
	lq.xyzw	fVelocity,	(iVelocity)	; dx, dy
	
	move.xyzw	fNewVelocity,	fVelocity	;in case need update pri velocity

	; ------------- First Vertex ---------------------
	apply_velocity iCoord,fCoord1,fScrCenter,fVelocity
	bound_chk fCoord1, 210, 0x1, bouncex, checknegx
bouncex:  						; hit an x boundary
	abs.x	fNewVelocity,	fNewVelocity
	mul.x 		fNewVelocity, fNewVelocity, fXminus1	; reverse direction:  dx=  (-1.0)*dx
	b checky
bouncexb:  						; hit an x boundary
	abs.x	fNewVelocity,	fNewVelocity
	b checky
checknegx:
	fcand		vi01,0x2 			; check -x
	ibgtz		vi01, bouncexb
checky:
	bound_chk fCoord, 105, 0x04, bouncey, checknegy
bouncey:  						; hit a y boundary
	abs.y	fNewVelocity,	fNewVelocity
	mul.y 		fNewVelocity, fNewVelocity, fYminus1   	; reverse direction: dy=  (-1.0)*dy
	b render
bounceyb:
	abs.y	fNewVelocity,	fNewVelocity
	b render
checknegy:
	fcand		vi01, 0x08 			; check -y
	ibgtz		vi01, bounceyb

render:
	; Convert to Screen Coordinates
	add.xyz		fTemp,	fCoord1, fScrCenter
	ftoi4.xyz	fTemp, fTemp
	sq.xyz		fTemp, (iCoord)	; Update the screen coordinate

	; --------------- Second Vertex ------------------------
	iaddiu	iCoord, iCoord,	2		; next vertex

	apply_velocity iCoord,fCoord2,fScrCenter,fVelocity
	bound_chk fCoord2, 210, 0x1, bouncex2, checknegx2
bouncex2:  						; hit an x boundary
	abs.x	fNewVelocity,	fNewVelocity
	mul.x 		fNewVelocity, fNewVelocity,fXminus1	; reverse direction:  dx=  (-1.0)*dx
	b checky2
bouncex2b:  						; hit an x boundary
	abs.x	fNewVelocity,	fNewVelocity
	b checky2
checknegx2:
	fcand		vi01,0x2 			; check -x
	ibgtz		vi01, bouncex2b
checky2:
	bound_chk fCoord2, 105, 0x04, bouncey2, checknegy2
bouncey2:  						; hit a y boundary
	abs.y	fNewVelocity,	fNewVelocity	
	mul.y 		fNewVelocity,fNewVelocity,fYminus1   	; reverse direction: dy=  (-1.0)*dy
	b render2
bouncey2b:
	abs.y	fNewVelocity,	fNewVelocity
	b render2
checknegy2:
	fcand		vi01, 0x08 			; check -y
	ibgtz		vi01, bouncey2b

render2:
	; Convert to Screen Coordinates
	add.xyz		fTemp,	fCoord2, fScrCenter
	ftoi4.xyz	fTemp, fTemp
	sq.xyz		fTemp, (iCoord)	; Update the screen coordinate

	; --------------- Third Vertex ------------------------
	iaddiu	iCoord, iCoord,	2		; next vertex

	apply_velocity iCoord,fCoord3,fScrCenter,fVelocity
	bound_chk fCoord3, 210, 0x1, bouncex3, checknegx3
bouncex3:  							; hit an x boundary
	abs.x		fNewVelocity, fNewVelocity
	mul.x 		fNewVelocity, fNewVelocity, fXminus1	; reverse direction:  dx=  (-1.0)*dx
	b checky3
bouncex3b:  						; hit an x boundary
	abs.x	fNewVelocity,	fNewVelocity
	b checky3
checknegx3:
	fcand		vi01,0x2 			; check -x
	ibgtz		vi01, bouncex3b
checky3:
	bound_chk fCoord3, 105, 0x04, bouncey3, checknegy3
bouncey3:  						; hit a y boundary
	abs.y	fNewVelocity,	fNewVelocity
	mul.y 		fNewVelocity,fNewVelocity,fYminus1   	; reverse direction: dy=  (-1.0)*dy
	b render3
bouncey3b:
	abs.y	fNewVelocity,	fNewVelocity
	b render3
checknegy3:
	fcand		vi01, 0x08 			; check -y
	ibgtz		vi01, bouncey3b
render3:
	; Convert to Screen Coordinates
	add.xyz		fTemp,	fCoord3, fScrCenter
	ftoi4.xyz	fTemp, fTemp
	sq.xyz		fTemp, (iCoord)	; Update the screen coordinate

	; ============== Triangle 2: (-x, y), (-dx, dy) ====================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fXminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fXminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fXminus1

	; ============== Triangle 3: (x, -y) dx, -dy ====================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fYminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fYminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fYminus1

	; ============== Triangle 4: (-x, -y) -dx, -dy ====================	
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fXYminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fXYminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fXYminus1

	; swap (x,y), (dx, dy)
	swap_xy	fVelocity
	swap_xy	fCoord1
	swap_xy fCoord2
	swap_xy fCoord3
	
	mul	fVelocity, fVelocity, fXdoubleYhalf
	mul	fCoord1,  fCoord1, fXdoubleYhalf
	mul	fCoord2,  fCoord2, fXdoubleYhalf
	mul	fCoord3,  fCoord3, fXdoubleYhalf	

	; ============== Triangle 5: (y,x) (dy, dx) =========================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fPlus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fPlus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fPlus1
	
	; ============== Triangle 6: (-y, x) (-dy, dx) ======================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fXminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fXminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fXminus1

	; ============== Triangle 7: (y, -x) (dy, -dx) ======================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fYminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fYminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fYminus1

	; ============== Triangle 8: (-y, -x) (-dy, -dx) ======================
	move_vertex iCoord, fCoord1, fScrCenter, fVelocity, fXYminus1
	move_vertex iCoord, fCoord2, fScrCenter, fVelocity, fXYminus1
	move_vertex iCoord, fCoord3, fScrCenter, fVelocity, fXYminus1

primary_done:

	; Update Primary Velocity after all 8 triangless have been moved
	mul.xyz		fNewVelocity,	fNewVelocity,	fDamp    ; Gradually stop moving
	sq.xyz		fNewVelocity, (iVelocity) ; Update Velocity	
	iaddiu	iCoord, iCoord, 2
	iaddiu	iVelocity, iVelocity, 1		;next primary velocity	
	isubiu iCounter0, iCounter0, 1
	ibne	iCounter0, vi00, move_pri_tri

	iaddiu iKick, vi00, kGifPacket
	xgkick iKick

--exit
--endexit




