
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "defines.h"
#include "packet.h"
#include "geommath.h"

#include "texture.h"

#include "model.h"

#define NUM_CUBES 100 //ATM must be even
u_long128 packet_data[1024];
qword_t QW;
gifpacket_t gifpacket;
int prim;
int i,j;

inline float frand(float scale=1.0f)
{
	return scale * ((float)rand())/((float)RAND_MAX);
}

Mat44 makePerspectiveMatrix()
{
	Mat44 perspective;

	float n = 0.1f;
	float f = 1000.0f;
	float l = -0.06f * (2048.0f / (640.0f * 0.5f));
	float r =  0.06f * (2048.0f / (640.0f * 0.5f));
	float t =  0.06f * (2048.0f / (350.0f * 0.5f));
	float b = -0.06f * (2048.0f / (350.0f * 0.5f));

	LoadIdentity(&perspective);
	SetRow(&perspective,0,Vec4((2*n) / (r-l)	, 0.0f				, 0.0f				, 0.0f));
	SetRow(&perspective,1,Vec4(0.0f				, -(2*n) / (t-b)	, 0.0f				, 0.0f));
	SetRow(&perspective,2,Vec4(((r+l) / (r-l))	, ((t+b) / (t-b))	, ((f+n) / (f-n))	, -1.0f));
	SetRow(&perspective,3,Vec4(0.0f				, 0.0f				, ((2*f*n) / (f-n))	, 0.0f));
	return (perspective);
}

void add_texture_upload()
{
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(4, 0, 0, 0, GIF_PACKED, 1));
	gifpacket_addgsdata(&gifpacket, 0x0e);
	QW.u64[0] = GS_SET_BITBLTBUF(0,1,PSMCT16,0x118 << 5,1,PSMCT16);
	QW.u64[1] = BITBLTBUF_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TRXPOS(0,0,0,0,0);
	QW.u64[1] = TRXPOS_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TRXREG(64,64);
	QW.u64[1] = TRXREG_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TRXDIR(TRXDIR_TO_GS);
	QW.u64[1] = TRXDIR_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(512, 0, 0, 0, GIF_IMAGE, 0));
	gifpacket_addgsdata(&gifpacket, 0x0);
	for (i=0, j=0; i<(NUM_TEX_BYTES/24); i++)
	{
		//Convert from 24-bit to 16-bit colour.
		int b0 = ((tex[j++]) >> 3) << 10;
		int g0 = ((tex[j++]) >> 3) << 5;
		int r0 = ((tex[j++]) >> 3);
		int a0 = 0 << 15;
		int b1 = ((tex[j++]) >> 3) << 26;
		int g1 = ((tex[j++]) >> 3) << 21;
		int r1 = ((tex[j++]) >> 3) << 16;
		int a1 = 0 << 31;
		int b2 = ((tex[j++]) >> 3) << 10;
		int g2 = ((tex[j++]) >> 3) << 5;
		int r2 = ((tex[j++]) >> 3);
		int a2 = 0 << 15;
		int b3 = ((tex[j++]) >> 3) << 26;
		int g3 = ((tex[j++]) >> 3) << 21;
		int r3 = ((tex[j++]) >> 3) << 16;
		int a3 = 0 << 31;
		int b4 = ((tex[j++]) >> 3) << 10;
		int g4 = ((tex[j++]) >> 3) << 5;
		int r4 = ((tex[j++]) >> 3);
		int a4 = 0 << 15;
		int b5 = ((tex[j++]) >> 3) << 26;
		int g5 = ((tex[j++]) >> 3) << 21;
		int r5 = ((tex[j++]) >> 3) << 16;
		int a5 = 0 << 31;
		int b6 = ((tex[j++]) >> 3) << 10;
		int g6 = ((tex[j++]) >> 3) << 5;
		int r6 = ((tex[j++]) >> 3);
		int a6 = 0 << 15;
		int b7 = ((tex[j++]) >> 3) << 26;
		int g7 = ((tex[j++]) >> 3) << 21;
		int r7 = ((tex[j++]) >> 3) << 16;
		int a7 = 0 << 31;
		if ((b0 | b1 | b2 | b3 | b4 | b5 | b6 | b7) != 0)
		{
			printf("\nBlue non-zero at i=%d j=%d\n",i,j);
		}
		QW.u32[0] = r0 | g0 | b0 | a0 | r1 | g1 | b1 | a1;
		QW.u32[1] = r2 | g2 | b2 | a2 | r3 | g3 | b3 | a3;
		QW.u32[2] = r4 | g4 | b4 | a4 | r5 | g5 | b5 | a5;
		QW.u32[3] = r6 | g6 | b6 | a6 | r7 | g7 | b7 | a7;
		gifpacket_addgspacked(&gifpacket, QW.u128);
	}
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(4, 1, 0, 0, GIF_PACKED, 1));
	gifpacket_addgsdata(&gifpacket, 0x0e);
	QW.u64[0] = GS_SET_TEXFLUSH();
	QW.u64[1] = TEXFLUSH_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TEXA(0x80, 0, 0x80);
	QW.u64[1] = TEXA_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TEX1(1,0,0,0,0,0,0);
	QW.u64[1] = TEX1_1_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
	QW.u64[0] = GS_SET_TEX0(0x118 << 5, 1, PSMCT16, 6, 6, 0, 0, 0, 0, 0, 0, 0);
	QW.u64[1] = TEX0_1_REG;
	gifpacket_addgspacked(&gifpacket, QW.u128);
}

int write_file(char * filename)
{
	FILE *output;
	output=fopen(filename, "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", filename);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	int bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, filename);

	return 0;
}

int main(int argc, char **argv)
{
	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	time_t timenow;
	time(&timenow);
	srand(timenow);

	for (i=0; i<1024; i++)
	{
		packet_data[i] = 0;
	}

	gifpacket_init(&gifpacket, packet_data);

	for (i=0; i<NUM_VERTS; i++)
	{
		gifpacket_addgsvec4(&gifpacket, &(texts[i]));
		gifpacket_addgsvec4(&gifpacket, &(colours[i]));
		gifpacket_addgsvec4(&gifpacket, &(verts[i]));
	}

	for (i=0; i<NUM_CUBES; i++)
	{
		QW.f32[0] = frand(6.4f) - 3.2f;
		QW.f32[1] = frand(6.4f) - 3.2f;
		QW.f32[2] = frand(6.4f) - 3.2f;
		QW.f32[3] = frand();
		gifpacket_addgspacked(&gifpacket, QW.u128);

		Vec3 axis;
		axis.x = frand() - 0.5f;
		axis.y = frand() - 0.5f;
		axis.z = frand() - 0.5f;
		axis = Normalize(axis);
		QW.f32[0] = axis.x;
		QW.f32[1] = axis.y;
		QW.f32[2] = axis.z;
		QW.f32[3] = frand(M_PI * 2);
		gifpacket_addgspacked(&gifpacket, QW.u128);
	}

	Mat44 objectToWorld;
	LoadIdentity(&objectToWorld);
	gifpacket_addgsmat44(&gifpacket, &objectToWorld);

	Mat44 worldToCamera;
	LoadIdentity(&worldToCamera);
	//SetRow(&worldToCamera, 3, Vec4(0,0,-5.0f,1));
	gifpacket_addgsmat44(&gifpacket, &worldToCamera);

	Mat44 cameraToScreen = makePerspectiveMatrix();
	gifpacket_addgsmat44(&gifpacket, &cameraToScreen);

	QW.f32[0] = 2048.0f;
	QW.f32[1] = 2048.0f;
	QW.f32[2] = (float)((2<<24)-1)/2;
	QW.f32[3] = 1.0f;
	gifpacket_addgspacked(&gifpacket,QW.u128);

	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(0, 1, 0, 0, 0, 0));
	gifpacket_addgsdata(&gifpacket, 0x0);

	prim=GS_SET_PRIM(GS_PRIM_TRI, 0, 1, 0, 0, 0, 0, 0, 0);
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NUM_VERTS*2, 1, 1, prim, GIF_PACKED, 3));
	gifpacket_addgsdata(&gifpacket, 0x512);

	add_texture_upload();

	gifpacket_terminate(&gifpacket);

	return write_file(argv[1]);
}
