
#include <stdio.h>
#include "defines.h"
#include "packet.h"
#include "geommath.h"
#include <math.h>

#include "texture.h"

#include "model.h"

Mat44 makePerspectiveMatrix()
{
	Mat44 perspective;

	float n = 0.1f;
	float f = 1000.0f;
	float l = -0.06f * (2048.0f / (640.0f * 0.5f));
	float r =  0.06f * (2048.0f / (640.0f * 0.5f));
	float t =  0.06f * (2048.0f / (350.0f * 0.5f));
	float b = -0.06f * (2048.0f / (350.0f * 0.5f));

	LoadIdentity(&perspective);
	SetRow(&perspective,0,Vec4((2*n) / (r-l)	, 0.0f				, 0.0f				, 0.0f));
	SetRow(&perspective,1,Vec4(0.0f				, -(2*n) / (t-b)	, 0.0f				, 0.0f));
	SetRow(&perspective,2,Vec4(((r+l) / (r-l))	, ((t+b) / (t-b))	, ((f+n) / (f-n))	, -1.0f));
	SetRow(&perspective,3,Vec4(0.0f				, 0.0f				, ((2*f*n) / (f-n))	, 0.0f));
	return (perspective);
}

int main(int argc, char **argv)
{
	FILE *output;
	int bytes=0;
	int prim=0;
	qword_t colour;
	qword_t scales;
	qword_t reg;
	int i,j;

	u_long128 packet_data[1024];

	gifpacket_t gifpacket;

	for (i=0; i<1024; i++)
	{
		packet_data[i] = 0;
	}

	if (argc!=2)
	{
		fprintf(stderr, "Usage: gen_data output_file\n");
		return -1;
	}

	gifpacket_init(&gifpacket, packet_data);

	for (i=0; i<NUM_VERTS; i++)
	{
		gifpacket_addgsvec4(&gifpacket, &(texts[i]));
		gifpacket_addgsvec4(&gifpacket, &(verts[i]));
	}

	prim=GS_SET_PRIM(GS_PRIM_TRI, GOURAUD(1), TEXTURED(1), FOG(0), ABE(0), ALPHA(0), AA(0), CTXT(0), FIX(0));
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(NUM_VERTS), EOP(1), PRE(1), prim, GIF_PACKED, NREG(3)));
	gifpacket_addgsdata(&gifpacket, 0x512);

	for (i=0; i<NUM_VERTS; i++)
	{
		colour.u32[0] = 255;
		colour.u32[1] = 255;
		colour.u32[2] = 255;
		colour.u32[3] = 0x80;
		gifpacket_addgspacked(&gifpacket, 0); // Leave a gap for transformed STQ co-ords
		gifpacket_addgspacked(&gifpacket, colour.u128);
		gifpacket_addgspacked(&gifpacket, 0); // Leave a gap for transformed vertex co-ords
	}

	Mat44 objectToWorld;
	LoadIdentity(&objectToWorld);
	gifpacket_addgsmat44(&gifpacket, &objectToWorld);

	Mat44 worldToCamera;
	LoadIdentity(&worldToCamera);
	SetRow(&worldToCamera, 3, Vec4(0,0,-5.0f,1));
	gifpacket_addgsmat44(&gifpacket, &worldToCamera);

	Mat44 cameraToScreen = makePerspectiveMatrix();
	gifpacket_addgsmat44(&gifpacket, &cameraToScreen);

	Mat44 rotate;
	LoadRotation(&rotate, 0.1f, 0.05f, 0.025f);
	gifpacket_addgsmat44(&gifpacket, &rotate);

	scales.f32[0] = 2048.0f;
	scales.f32[1] = 2048.0f;
	scales.f32[2] = (float)((2<<24)-1)/2;
	scales.f32[3] = 1.0f;
	gifpacket_addgspacked(&gifpacket,scales.u128);

	//Put texture and texture upload info here
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(4), EOP(0), PRE(0), 0, GIF_PACKED, NREG(1)));
	gifpacket_addgsdata(&gifpacket, 0x0e);
	//1) Set Destination buffer in BITBLTBUF (0x118 is first free address in GS ram)
	//Bugfix - Address is specified in 256 byte units, i.e. 0x118 * 32
	reg.u64[0] = GS_SET_BITBLTBUF(0,1,PSMCT16,0x118 << 5,1,PSMCT16);
	reg.u64[1] = BITBLTBUF_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	//2) Set Destination position in TRXPOS (see harness source for address)
	reg.u64[0] = GS_SET_TRXPOS(0,0,0,0,0);
	reg.u64[1] = TRXPOS_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	//3) Set Destination transmission area in TRXREG
	reg.u64[0] = GS_SET_TRXREG(64,64);
	reg.u64[1] = TRXREG_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	//4) Kick off Transfer with TRXDIR
	reg.u64[0] = GS_SET_TRXDIR(TRXDIR_TO_GS);
	reg.u64[1] = TRXDIR_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	//5) Supply IMAGE mode GIFtag for the texture transfer
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(512), EOP(0), PRE(0), 0, GIF_IMAGE, NREG(0)));
	gifpacket_addgsdata(&gifpacket, 0x0);
	for (i=0, j=0; i<(NUM_TEX_BYTES/24); i++)
	{
		//Convert from 24-bit to 16-bit colour.
		int b0 = ((tex[j++]) >> 3) << 10;
		int g0 = ((tex[j++]) >> 3) << 5;
		int r0 = ((tex[j++]) >> 3);
		int a0 = 0 << 15;
		int b1 = ((tex[j++]) >> 3) << 26;
		int g1 = ((tex[j++]) >> 3) << 21;
		int r1 = ((tex[j++]) >> 3) << 16;
		int a1 = 0 << 31;
		int b2 = ((tex[j++]) >> 3) << 10;
		int g2 = ((tex[j++]) >> 3) << 5;
		int r2 = ((tex[j++]) >> 3);
		int a2 = 0 << 15;
		int b3 = ((tex[j++]) >> 3) << 26;
		int g3 = ((tex[j++]) >> 3) << 21;
		int r3 = ((tex[j++]) >> 3) << 16;
		int a3 = 0 << 31;
		int b4 = ((tex[j++]) >> 3) << 10;
		int g4 = ((tex[j++]) >> 3) << 5;
		int r4 = ((tex[j++]) >> 3);
		int a4 = 0 << 15;
		int b5 = ((tex[j++]) >> 3) << 26;
		int g5 = ((tex[j++]) >> 3) << 21;
		int r5 = ((tex[j++]) >> 3) << 16;
		int a5 = 0 << 31;
		int b6 = ((tex[j++]) >> 3) << 10;
		int g6 = ((tex[j++]) >> 3) << 5;
		int r6 = ((tex[j++]) >> 3);
		int a6 = 0 << 15;
		int b7 = ((tex[j++]) >> 3) << 26;
		int g7 = ((tex[j++]) >> 3) << 21;
		int r7 = ((tex[j++]) >> 3) << 16;
		int a7 = 0 << 31;
		colour.u32[0] = r0 | g0 | b0 | a0 | r1 | g1 | b1 | a1;
		colour.u32[1] = r2 | g2 | b2 | a2 | r3 | g3 | b3 | a3;
		colour.u32[2] = r4 | g4 | b4 | a4 | r5 | g5 | b5 | a5;
		colour.u32[3] = r6 | g6 | b6 | a6 | r7 | g7 | b7 | a7;
		gifpacket_addgspacked(&gifpacket, colour.u128);
	}
	//512 QWords for a 64*64 texture... that's half my memory!  Good job it can be claimed back...
	//6) Access the TEXFLUSH register
	gifpacket_addgsdata(&gifpacket, GIF_SET_TAG(NLOOP(4), EOP(1), PRE(0), 0, GIF_PACKED, NREG(1)));
	gifpacket_addgsdata(&gifpacket, 0x0e);
	reg.u64[0] = GS_SET_TEXFLUSH();
	reg.u64[1] = TEXFLUSH_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	//7) Set TEXA, TEX1, TEX0, MIPTBP1, MIPTBP2
	reg.u64[0] = GS_SET_TEXA(0x80, 0, 0x80);
	reg.u64[1] = TEXA_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	reg.u64[0] = GS_SET_TEX1(1,0,0,0,0,0,0);
	reg.u64[1] = TEX1_1_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);
	reg.u64[0] = GS_SET_TEX0(0x118 << 5, 1, PSMCT16, 6, 6, 0, 0, 0, 0, 0, 0, 0);
	reg.u64[1] = TEX0_1_REG;
	gifpacket_addgspacked(&gifpacket, reg.u128);

	gifpacket_terminate(&gifpacket);

	output=fopen(argv[1], "wb");

	if (output==NULL)
	{
		fprintf(stderr, "Could not open file \"%s\"\n", argv[1]);
		return -1;
	}

	fwrite(packet_data, sizeof(packet_data)/sizeof(qword_t), sizeof(qword_t), output);

	bytes=ftell(output);
	fclose(output);

	printf("Written %d bytes to \"%s\"\n", bytes, argv[1]);

	return 0;
}
